/*
 * Copyright (c) 2003 Chris Wareham
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Chris Wareham.
 * 4. The name of Chris Wareham may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CHRIS WAREHAM ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
 * EVENT SHALL CHRIS WAREHAM BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef __NetBSD__

#include <sys/param.h>
#include <sys/sysctl.h>

#include <kvm.h>
#include <limits.h>
#include <signal.h>
#include <unistd.h>

/*
 * Sends signal "signum" to all processes that match "progname" and have the
 * effective UID of the caller  Returns the number of processes successfully
 * signalled or -1 on error.
 */
int
pkill(const char *progname, int signum)
{
	uid_t uid;
	kvm_t *kd;
	char buf[_POSIX2_LINE_MAX];
	struct kinfo_proc2 *list, *kp;
	int nproc, i, retval;

	uid = getuid();

	kd = kvm_openfiles(NULL, NULL, NULL, KVM_NO_FILES, buf);
	if (kd == NULL)
		return -1;

	list = kvm_getproc2(kd, KERN_PROC_UID, uid, sizeof(*list), &nproc);
	if (list == NULL)
		return -1;

	for (retval = 0, i = 0, kp = list; i < nproc; i++, kp++) {
		if ((kp->p_flag & P_SYSTEM) != 0)
				continue;
		if (strcmp(kp->p_comm, progname) == 0) {
			if (kill(kp->p_pid, signum) == 0)
				retval++;
		}
	}

	return retval;
}

#else

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

static int ilen(int);

int
pkill(const char *progname, int signum)
{
	uid_t uid;
	char *fmt = "pkill -u %d -signal %d %s", *ptr;

	uid = getuid();

	ptr = malloc(strlen(fmt) + strlen(progname) + ilen(uid) + ilen(signum) + 1);
	sprintf(ptr, fmt, getuid(), signum, progname);
	system(ptr);
	free(ptr);

	return 0;
}

static int
ilen(int n)
{
	int i = 1;

	if (n < 0) {
		n = -n;
		i++;
	}

	while (n > 9) {
		n /= 10;
		i++;
	}

	return i;
}

#endif
